-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio                   "EskaQuestTracker.Classes.Block"                    ""
-- ========================================================================== --
namespace "EQT"
import "System"
-- ========================================================================== --
class "Block" inherit "Frame"
  event "OnActiveChanged"

  _BlockCache = setmetatable( {}, { __mode = "k"})
  -- ======================================================================== --
  -- Handlers
  -- ======================================================================== --
  function SetText(self, new, old, prop)
    local class = System.Reflector.GetObjectClass(self)

    if class.GetBlockProperty then
      local transform = class:GetBlockProperty("headerTextTransform")

      local txt = new
      if transform == "uppercase" then
        txt = txt:upper()
      elseif transform == "lowercase" then
        txt = txt:lower()
      end
      self.frame.header.text:SetText(txt)
    else
      self.frame.header.text:SetText(new)
    end
  end

  function Refresh(self)
    local class = System.Reflector.GetObjectClass(self)
    local color

    -- background color
    color = class:GetBlockProperty("backdropColor")
    self.frame:SetBackdropColor(color.r, color.g, color.b, color.a)
    -- border color
    color = class:GetBlockProperty("borderColor")
    self.frame:SetBackdropBorderColor(color.r, color.g, color.b, color.a)

    -- ## HEADER PART
    -- header background color
    color = class:GetBlockProperty("headerBackdropColor")
    self.frame.header:SetBackdropColor(color.r, color.g, color.b, color.a)
    -- header border color
    color = class:GetBlockProperty("headerBorderColor")
    self.frame.header:SetBackdropBorderColor(color.r, color.g, color.b, color.a)
    -- header stripe color
    color = class:GetBlockProperty("headerStripeColor")
    self.frame.header.stripe:SetVertexColor(color.r, color.g, color.b, color.a)


    -- header text properties
    local size = class:GetBlockProperty("headerTextSize")
    local font = _LibSharedMedia:Fetch("font", class:GetBlockProperty("headerTextFont"))

    local transform = class:GetBlockProperty("headerTextTransform")
    color = class:GetBlockProperty("headerTextColor")

    self.frame.header.text:SetFont(font, size, "OUTLINE")
    self.frame.header.text:SetTextColor(color.r, color.g, color.b, color.a)

    -- header text
    if class:IsBlockPropertyEnabled("headerText") then
      self.text = class:GetBlockProperty("headerText")
    else
      self.text = nil
    end

    local txt = self.text
    if transform == "uppercase" then
      txt = txt:upper()
    elseif transform == "lowercase" then
      txt = txt:lower()
    end
    self.frame.header.text:SetText(txt)

    -- header text location
    local location = class:GetBlockProperty("headerTextLocation")
    local offsetX = class:GetBlockProperty("headerTextOffsetX")
    local offsetY = class:GetBlockProperty("headerTextOffsetY")
    self.frame.header.text:ClearAllPoints()
    self.frame.header.text:SetPoint(location, offsetX, offsetY)

  end

  __Static__() function RefreshAll()
    for obj in pairs(_BlockCache) do
      obj:Refresh()
    end
  end
  -- ======================================================================== --
  -- Properties
  -- ======================================================================== --
  property "id" { TYPE = String }
  property "text" { TYPE = String, DEFAULT = "Default Header Text", HANDLER = "SetText"}
  property "isActive" { TYPE = Boolean, DEFAULT = true, EVENT = "OnActiveChanged" }
  property "priority" { TYPE = Number, DEFAULT = 100 }

  __Static__() property "backgroundColor" {
    TYPE = Table,
    SET = function(self, color) _DB.Block.backdropColor = color ; self:RefreshAll() end,
    GET = function() return _DB.Block.backdropColor  end,
  }

  __Static__() property "borderColor" {
    TYPE = Table,
    SET = function(self, color) _DB.Block.borderColor = color ; self:RefreshAll() end,
    GET = function() return _DB.Block.borderColor end,
  }

  __Static__()  property "headerStripeColor" {
    TYPE = Table,
    SET = function(self, color)
      _DB.Block.headerStripeColor = color
      self:RefreshAll()
    end,
    GET = function() return _DB.Block.headerStripeColor end

  }

  __Static__() property "headerBackdropColor" {
    TYPE = Table,
    SET = function(self, color) _DB.Block.headerBackdropColor = color ; self:RefreshAll() end,
    GET = function() return _DB.Block.headerBackdropColor end,
  }

  __Static__() property "headerBorderColor" {
    TYPE = Table,
    SET = function(self, color) _DB.Block.headerBorderColor = color ; self:RefreshAll() end,
    GET = function() return _DB.Block.headerBorderColor end,
  }

  __Static__() property "headerTextSize" {
    TYPE = Number,
    SET = function(self, size) _DB.Block.headerTextSize = size ; self:RefreshAll() end,
    GET = function() return _DB.Block.headerTextSize end,
  }

  __Static__() property "headerTextFont" {
    TYPE = String,
    SET = function(self, font) _DB.Block.headerTextFont = font ; self:RefreshAll() end,
    GET = function() return _DB.Block.headerTextFont end,
  }

  __Static__() property "headerTextColor" {
    TYPE = Table,
    SET = function(self, color) _DB.Block.headerTextColor = color ; self:RefreshAll() end,
    GET = function() return _DB.Block.headerTextColor end,
  }

  __Static__() property "headerTextTransform" {
    TYPE = String,
    SET = function(self, transform) _DB.Block.headerTextTransform = transform ; self:RefreshAll() end,
    GET = function() return _DB.Block.headerTextTransform end,
  }

  __Static__() property "headerTextOffsetX" {
    TYPE = Number,
    SET = function(self, offsetX) _DB.Block.headerTextOffsetX = offsetX ; self:RefreshAll() end,
    GET = function() return _DB.Block.headerTextOffsetX end,
  }

  __Static__() property "headerTextOffsetY" {
    TYPE = Number,
    SET = function(self, offsetY) _DB.Block.headerTextOffsetY = offsetY ; self:RefreshAll() end,
    GET = function() return _DB.Block.headerTextOffsetY end,
  }

  __Static__() property "headerTextLocation" {
    TYPE = String,
    SET = function(self, anchorPoint) _DB.Block.headerTextLocation = anchorPoint ; self:RefreshAll() end,
    GET = function() return _DB.Block.headerTextLocation end
  }
  -- ======================================================================== --
  -- Constructors
  -- ======================================================================== --
  __Arguments__ { }
  function Block(self)
    Super(self)

    local frame = CreateFrame("Frame")
    frame:SetBackdrop(_Backdrops.CommonWithBiggerBorder)
    frame:SetBackdropColor(0, 0, 0, 0)
    frame:SetBackdropBorderColor(0, 0, 0, 0)

    local headerFrame = CreateFrame("Frame", nil, frame)
    headerFrame:SetPoint("TOPLEFT")
    headerFrame:SetPoint("TOPRIGHT", 2, 0)
    headerFrame:SetFrameStrata("HIGH")
    headerFrame:SetHeight(34) -- 24
    headerFrame:SetBackdrop(_Backdrops.CommonWithBiggerBorder)
    headerFrame:SetBackdropBorderColor(0, 0, 0)
    --headerFrame:SetBackdropColor(0.4, 0.4, 0.4, 0) -- 0.4 0.4 0.4 1
    headerFrame:SetBackdropColor(0, 0, 0, 0.5)
    frame.header = headerFrame

    local bg = headerFrame:CreateTexture()
    bg:SetAllPoints()
    bg:SetColorTexture(0, 0, 0, 1)
    bg:SetDrawLayer("BACKGROUND", 1)
    bg:Hide()
    headerFrame.bg = bg

    local stripe = headerFrame:CreateTexture()
    stripe:SetAllPoints()
    stripe:SetTexture([[Interface\AddOns\EskaQuestTracker\Media\Textures\Stripe]])
    stripe:SetDrawLayer("ARTWORK", 2)
    stripe:SetBlendMode("ALPHAKEY")
    stripe:SetVertexColor(0, 0, 0, 0.5)
    headerFrame.stripe = stripe

    local font = _LibSharedMedia:Fetch("font", "PT Sans Narrow Bold")

    local headerText = headerFrame:CreateFontString(nil, "OVERLAY")
    headerText:SetPoint("CENTER", -10, 20)
    headerText:SetFont(font, 17, "OUTLINE") -- 12
    headerText:SetText(self.text:upper())
    headerText:SetTextColor(0, 199/255, 1)
    headerText:SetShadowColor(0, 0, 0, 0.25)
    headerText:SetShadowOffset(1, -1)
    headerFrame.text = headerText

    self.frame = frame
    self.height = 34
    self.baseHeight = self.height

    _BlockCache[self] = true

  end

  __Arguments__{ String, Number }
  function Block(self, id, priority)
    This(self)

    self.id = id
    self.priority = priority
  end

endclass "Block"
-- ========================================================================== --
-- == OnLoad Handler
-- ========================================================================== --
function OnLoad(self)
  _DB:SetDefault("Block", {
    -- frame color
    backdropColor = { r = 0, g = 0, b = 0, a = 0},
    borderColor = { r = 0, g = 0, b = 0, a = 0 },
    -- header Color
    headerStripeColor = { r = 0, g = 0, b = 0, a = 0.5 },
    headerBackdropColor = { r = 0, g = 0, b = 0, a = 0.5 },
    headerBorderColor = { r = 0, g = 0, b = 0, a = 1},
    -- header Text properties
    headerTextSize = 17,
    headerTextColor = { r = 0, g = 199/255, b = 1},
    headerTextFont = "PT Sans Narrow Bold",
    headerTextTransform = "uppercase",
    headerTextOffsetX = 0,
    headerTextOffsetY = 0,
    headerTextLocation = "CENTER",
  })
end
