-- ========================================================================== --
-- 										 EskaQuestTracker                                       --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/eska-quest-tracker         --
-- ========================================================================== --
Scorpio         "EskaQuestTracker.Classes.Dungeon"                            ""
-- ========================================================================== --
namespace "EQT"
-- ========================================================================== --
function OnLoad(self)
    _DB:SetDefault("Dungeon", {
    textSizes = {
      name = 12,
    },
    textFonts = {
      name = "PT Sans Narrow Bold",
    },
    textTransforms = {
      name = "uppercase",
    },
    textColors = {
      name = { r = 1, g = 0.5, b = 0},
    }
  })
end
-- ========================================================================== --
__InitChildBlockDB__()
class "Dungeon" inherit "Block" extend "IObjectiveHolder"
  _DungeonCache = setmetatable( {}, { __mode = "k" } )
  -- ======================================================================== --
  -- Handlers
  -- ======================================================================== --
  local function SetName(self, new, old, prop)
    self.frame.name:SetText(new)
  end

  local function SetTexture(self, new, old)
    self.frame.ftex.texture:SetTexture(new)
  end
  -- ======================================================================== --
  -- Methods                                                                  --
  -- ======================================================================== --
  function Draw(self)
    if self.numObjectives > 0 then
      local obj = self.objectives[1]
      if obj then
        obj.frame:SetPoint("TOPLEFT", self.frame.ftex, "TOPRIGHT")
        obj.frame:SetPoint("TOPRIGHT", self.frame.header, "BOTTOMRIGHT")
        self:DrawObjectives(self.frame, true)
        self.height = self.height + 5
      end
    else
      self.height = self.baseHeight + 92 + 8
    end

    if self.height < self.baseHeight + 92 then
      self.height = self.baseHeight + 92 + 8
    end
  end


  function Refresh(self)
    Super.Refresh(self)

    local font = _LibSharedMedia:Fetch("font", _DB.Dungeon.textFonts["name"])
    --local size = _DB.Dungeon.textSizes["name"]
    local color = _DB.Dungeon.textColors["name"]
    local transform = _DB.Dungeon.textTransforms["name"]

    local txt = self.name
    --if transform == "uppercase" then
      --txt = txt:upper()
    --elseif transform == "lowercase" then
      --txt = txt:lower()
    --end

    self.frame.name:SetFont(font, 17, "OUTLINE")
    self.frame.name:SetTextColor(color.r, color.g, color.b)
    self.frame.name:SetText(txt)
  end

  __Static__()
  function RefreshAll()
    for obj in pairs(_DungeonCache) do
      obj:Refresh()
    end
  end

  -- ======================================================================== --
  -- Properties
  -- ======================================================================== --
  property "name" { TYPE = String, DEFAULT = "", HANDLER = SetName }
  property "texture" { TYPE = String + Number, DEFAULT = nil, HANDLER = SetTexture}
  property "text" { TYPE = String, DEFAULT = "Dungeon", HANDLER = "SetText" }
  -- ======================================================================== --
  -- Constructors
  -- ======================================================================== --
  function Dungeon(self)
    Super(self, "Dungeon", 10)
    self.text = "Dungeon"

    local header = self.frame.header
    local headerText = header.text

    self.frame:SetBackdropColor(0, 0, 0, 0.3)
    self.frame:SetBackdropBorderColor(0, 0, 0, 1)

    -- Dungeon name
    local name = header:CreateFontString(nil, "OVERLAY")
    name:SetPoint("TOPRIGHT")
    name:SetPoint("BOTTOMRIGHT")
    name:SetPoint("LEFT", headerText, "RIGHT")
    name:SetJustifyH("CENTER")
    self.frame.name = name

    -- Set the headerText to left
    headerText:SetPoint("LEFT")
    headerText:SetJustifyH("LEFT")
    headerText:SetWidth(75)

    -- Dungeon Texture
    local ftex = CreateFrame("Frame", nil, self.frame)
    ftex:SetBackdrop(_Backdrops.CommonWithBiggerBorder)
    ftex:SetBackdropBorderColor(0, 0, 0, 0)
    ftex:SetBackdropColor(0, 0, 0)
    ftex:SetPoint("TOPLEFT", header, "BOTTOMLEFT", 4, -4)
    ftex:SetHeight(92)
    ftex:SetWidth(92)
    self.frame.ftex = ftex

    local texture = ftex:CreateTexture()
    texture:SetPoint("CENTER")
    texture:SetHeight(90)
    texture:SetWidth(90)
    texture:SetTexCoord(0.07, 0.93, 0.07, 0.93)
    ftex.texture = texture

    self.baseHeight = self.height

    self:Refresh()

    _DungeonCache[self] = true
  end
endclass "Dungeon"

function OnLoad(self)
  -- @HACK This is a temporary fix in waiting to release the theme API
  Dungeon.customConfigEnabled = true
  Dungeon:SetBlockPropertyValue("headerTextLocation", "LEFT")
end
